import unittest

from rdflib.namespace import XSD
from rdflib.term import _XSD_DOUBLE, Literal

from rdfdb.rdflibpatch_literal import _literal_n3, patch
patch()


class TestDoubleOutput(unittest.TestCase):

    def testNoDanglingPoint(self):
        vv = Literal("0.88", datatype=_XSD_DOUBLE)
        out = _literal_n3(vv, use_plain=True)
        print(out)
        self.assertTrue(out in ["8.8e-01", "0.88"], out)

    def testDoctestsFromRdflibTerm(self):
        # these include the changes my patch brings
        assert Literal(1)._literal_n3(use_plain=True) == '1'
        assert Literal(1.0)._literal_n3(use_plain=True) == '1.0'
        assert Literal(1.0, datatype=XSD.decimal)._literal_n3(use_plain=True) == '1.0'
        assert Literal(1.0, datatype=XSD.float)._literal_n3(use_plain=True) == '"1.0"^^<http://www.w3.org/2001/XMLSchema#float>'
        assert Literal("foo", datatype=XSD.string)._literal_n3(use_plain=True) == '"foo"^^<http://www.w3.org/2001/XMLSchema#string>'
        assert Literal(True)._literal_n3(use_plain=True) == 'true'
        assert Literal(False)._literal_n3(use_plain=True) == 'false'
        assert Literal(1.91)._literal_n3(use_plain=True) == '1.91'

        # Only limited precision available for floats:
        assert Literal(0.123456789)._literal_n3(use_plain=True) == '0.123456789'
        assert Literal('0.123456789', datatype=XSD.decimal)._literal_n3(use_plain=True) == '0.123456789'

        # Using callback for datatype QNames::

        assert Literal(1)._literal_n3(qname_callback=lambda uri: "xsd:integer") == '"1"^^xsd:integer'
